from decode import *
import decode

# http://en.wikipedia.org/wiki/OBD-II_PIDs
pids={
    1:{
0x00:(4, "PIDs supported [01 - 20]",
      decodeSuppordedPID,(0,)),
0x01:(4, "Monitor status since DTCs cleared.",
      decodeMotorStatus, ()),
0x02:(2, "Freeze DTC",
      None, None),
0x03:(2, "Fuel system status",
      decodeFuelStatus, ()),
0x04:(1, "Calculated engine load value",
      decodeInt, (1, 0, 100, "%")),
0x05:(1, "Engine coolant temperature",
      decodeInt, (1, -40, 215, "C")),
0x06:(1, "Short term fuel % trim-Bank 1",
      decodeInt, (1, -100, 127*100/128., "%")),
0x07:(1, "Long term fuel % trim-Bank 1",
      decodeInt, (1, -100, 127*100/128., "%")),
0x08:(1, "Short term fuel % trim-Bank 2",
      decodeInt, (1, -100, 127*100/128., "%")),
0x09:(1, "Long term fuel % trim-Bank 2",
      decodeInt, (1, -100, 127*100/128., "%")),
0x0A:(1, "Fuel pressure",
      decodeInt, (1, 0, 3*255, "Pa")),
0x0B:(1, "Intake manifold absolute pressure",
      decodeInt, (1, 0, 255, "kPa")),
0x0C:(2, "Engine RPM",
      decodeInt, (2, 0, (2**16-1.0)/4, "rpm")),
0x0D:(1, "Vehicle speed",
      decodeInt, (1, 0, 255, "km/h")),
0x0E:(1, "Timing advance",
      decodeInt, (1, -64, 63.5, "Deg to 1st cyl")),
0x0F:(1, "Intake air temperature",
      decodeInt, (1, -40, 215, "C")),
0x10:(2, "MAF air flow rate",
      decodeInt, (2, 0, (2**16-1)/100., "gr/s")),
0x11:(1, "Throttle position",
      decodeInt, (1, 0, 100, "%")),
0x12:(1, "Commanded secondary air status",
      decodeSecondaryAirStatus, ()),
0x13:(1, "Oxygen sensors present",
      decodeOxygenSensorsPresent, ()),
0x14:(2, "Bank 1, Sensor 1: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (1, 1)),
0x15:(2, "Bank 1, Sensor 2: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (1, 2)),
0x16:(2, "Bank 1, Sensor 3: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (1, 3)),
0x17:(2, "Bank 1, Sensor 4: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (1, 4)),
0x18:(2, "Bank 2, Sensor 1: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (2, 1)),
0x19:(2, "Bank 2, Sensor 2: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (2, 2)),
0x1A:(2, "Bank 2, Sensor 3: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (2, 3)),
0x1B:(2, "Bank 2, Sensor 4: Oxygen sensor voltage, Short term fuel trim",
      decodeOxygenSensorVoltage, (2, 4)),
0x1C:(1, "OBD standards this vehicle conforms to",
      decodeOBDStandards, ()),
0x1D:(1, "Oxygen seensors present",
      decodeOxygenSensorsPresent1D, ()),
0x1E:(1, "Auxiliary input status",
      decodeBits, (1,{0:"Power Take Off (PTO) status (1 == active)"})),
0x1F:(2, "Run time since engine start",
      decodeInt, (2, 0, 2**16-1, "s")),

0x20:(4, "PIDs supported [21 - 40]",
      decodeSuppordedPID,()),

0x21:(2, "Distance traveled with malfunction indicator lamp (MIL) on",
      decodeInt, (2, 0, 65535, "km")),
0x22:(2, "Fuel Rail Pressure (relative to manifold vacuum)",
      decodeInt, (2, 0,5177.265, "kPa")),
0x23:(2, "Fuel Rail Pressure (diesel, or gasoline direct inject)",
      decodeInt, (2, 0, 655350, "kPa")),
0x24:(4, "O2S1_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x25:(4, "O2S2_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x26:(4, "O2S3_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x27:(4, "O2S4_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x28:(4, "O2S5_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x29:(4, "O2S6_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x2A:(4, "O2S7_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x2B:(4, "O2S8_WR_lambda(1):Equivalence Ratio Voltage",
      decodeIntAr, (4, (0,0), (2,8), ("", "V"))),
0x2C:(1, "Commanded EGR",
      decodeInt, (1, 0,100, "%")),
0x2D:(1, "EGR Error",
      decodeInt, (1, -100, 99.22, "%")),
0x2E:(1, "Commanded evaporative purge",
      decodeInt, (1, 0, 100, "%")),
0x2F:(1, "Fuel Level Input",
      decodeInt, (1, 0, 100, "%")),
0x30:(1, "# of warm-ups since codes cleared",
      decodeInt, (1, 0, 255, "")),
0x31:(2, "Distance traveled since codes cleared", 
      decodeInt, (2, 0, 65535, "km")),
0x32:(2, "Evap. System Vapor Pressure", 
      decodeInt, (2, -8192, 8192, "Pa")), 
0x33:(1, "Barometric pressure", 
      decodeInt, (1, 0, 255, "kPa (Absolute)")),
0x34:(4, "O2S1_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x35:(4, "O2S2_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x36:(4, "O2S3_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x37:(4, "O2S4_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x38:(4, "O2S5_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x39:(4, "O2S6_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x3A:(4, "O2S7_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x3B:(4, "O2S8_WR_lambda(1): Equivalence Ratio Current",
      decodeIntAr, (4, (0,-128), (2,128), ("", "mA"))),
0x3C:(2, "Catalyst Temperature Bank 1, Sensor 1",
      decodeInt, (2, -40, 6513.5, "C")),
0x3D:(2, "Catalyst Temperature Bank 2, Sensor 1",
      decodeInt, (2, -40, 6513.5, "C")),
0x3E:(2, "Catalyst Temperature Bank 1, Sensor 2",
      decodeInt, (2, -40, 6513.5, "C")),
0x3F:(2, "Catalyst Temperature Bank 2, Sensor 2",
      decodeInt, (2, -40, 6513.5, "C")),


0x40:(4, "PIDs supported [41 - 60]",
      decodeSuppordedPID,()),

0x41:(4, "Monitor status this drive cycle",
      decodeMonitorStatusThisDriveCycle, ()),
0x42:(2, "Control module voltage",
      decodeInt, (2, 0, 65.535, "V")),
0x43:(2, "Absolute load value",
      decodeInt, (2, 0, 25700, "%")),
0x44:(2, "Command equivalence ratio",
      decodeInt, (2, 0, 2, "")),
0x45:(1, "Relative throttle position",
      decodeInt, (1, 0, 100, "%")),
0x46:(1, "Ambient air temperature",
      decodeInt, (1, -40, 215, "C")),
0x47:(1, "Absolute throttle position B",
      decodeInt, (1, 0, 100, "%")),
0x48:(1, "Absolute throttle position C",
      decodeInt, (1, 0, 100, "%")),
0x49:(1, "Accelerator pedal position D",
      decodeInt, (1, 0, 100, "%")),
0x4A:(1, "Accelerator pedal position E",
      decodeInt, (1, 0, 100, "%")),
0x4B:(1, "Accelerator pedal position F",
      decodeInt, (1, 0, 100, "%")),
0x4C:(1, "Commanded throttle actuator",
      decodeInt, (1, 0, 100, "%")),
0x4D:(2, "Time run with MIL on",
      decodeInt, (2, 0, 65535, "minutes")),
0x4E:(2, "Time since trouble codes cleared",
      decodeInt, (2, 0, 65535, "minutes")),
0x4F:(4, "Maximum value for equivalence ratio, oxygen sensor voltage, oxygen sensor current, and intake manifold absolute pressure",
      decodeIntAr, (4, (0, 0, 0, 0), (255, 255, 255, 2550), ("", "V", "mA", "kPa"))),
0x50:(4, "Maximum value for air flow rate from mass air flow sensor",
      decodeInt, (1, 0, 2550, "g/s")), #bytes 1, 2, 3 are reserved.
0x51:(1, "Fuel Type",
      decodeFuelType, ()),
0x52:(1, "Ethanol fuel %",
      decodeInt, (1, 0, 100, "%")),
0x53:(2, "Absolute Evap system Vapour Pressure",
      decodeInt, (2, 0, 327.675, "kPa")),
0x54:(2, "Evap system vapor pressure",
      decodeInt, (2, -32767, 32768, "Pa")),
0x55:(2, "Short term secondary oxygen sensor trim bank 1 and bank 3",
      decodeIntAr, (2, (-100, -100), (99.22, 99.22), ("%", "%"))),
0x56:(2, "Long term secondary oxygen sensor trim bank 1 and bank 3",
      decodeIntAr, (2, (-100, -100), (99.22, 99.22), ("%", "%"))),
0x57:(2, "Short term secondary oxygen sensor trim bank 2 and bank 4",
      decodeIntAr, (2, (-100, -100), (99.22, 99.22), ("%", "%"))),
0x58:(2, "Long term secondary oxygen sensor trim bank 2 and bank 4",
      decodeIntAr, (2, (-100, -100), (99.22, 99.22), ("%", "%"))),
0x59:(2, "Fuel rail pressure (absolute)",
      decodeInt, (2, 0, 655350, "kPa")),
0x5A:(1, "Relative accelerator pedal position",
      decodeInt, (1, 0, 100, "%")),
0x5B:(1, "Hybrid battery pack remaining life",
      decodeInt, (1, 0, 100, "%")),
0x5C:(1, "Engine oil temperature",
      decodeInt, (1, -40, 215, "%")),
0x5D:(2, "Fuel injection timing",
      decodeInt, (2, -210.00, 	301.992, "deg")),
0x5E:(2, "Engine fuel rate",
      decodeInt, (2, 0, 3212.75, "l/h")),
0x5F:(1, "Emission requirements to which vehicle is designed",
      decodeEmissionReqs, ())
}}

decodeSetPIDcodes(pids)
#print decodeSuppordedPID(hex2bin("BE 1F B8 11 "), 0)
#print decodeMotorStatus(hex2bin("84 07 61 00"), 0)
#print decodeFuelStatus(hex2bin("02 01"), 0)
#print decodeInt(hex2bin("80"), 4, 1, 0, 100, "%")

def checkCodes(pids):
    for mode in pids.keys():
        for pid in pids[mode].keys():
            data=pids[mode][pid]
            #print "DATA: ",data
            if len(data)!=4:
                print "hej, pid 0x%x: len of ar=%i != 4"%(pid, len(data))
            nbyte,name,func,args=data
            if func==decodeInt:
                if len(args)!=4:
                    print "pid=%x, func=decodeInt, but len(args)=%i!=4"%(pid, len(args))
                if args[0]!=nbyte:
                    print "pid=%x, func=decodeInt, args[0]=%i!=nbyte=%i"%(pid, args[0], nbyte)
            if func==decodeIntAr:
                if len(args)!=4:
                    print "pid=%x, func=decodeInt, but len(args)=%i!=4"%(pid, len(args))
    
if __name__ == "__main__":
    checkCodes(pids)
