
import serial
import subprocess  #only for test-mode
import thread
import time
import decode
import obdtrace
import codes
traceHex=obdtrace.obdtrace("hextrace.log")

class pipe:
    def __init__(self, name):
        self.p=subprocess.Popen(name,
                                stdin=subprocess.PIPE, 
                                stdout=subprocess.PIPE)
    def write(self, s):
        self.p.stdin.write(s)
    def read(self, n=None):
        return self.p.stdout.read(n)
    def flush(self):
        self.p.stdin.flush()

class elmComm:
    def __init__(self,
                 port="/dev/ttyUSB0",
                 baud=38400):
        self.elmState="READY"
        self.elmData=[]
        if port[0]!="|":
            self.ser = serial.Serial(port, #"/dev/ttyUSB0",
                                     baud, #38400,
                                     parity=serial.PARITY_NONE,
                                     stopbits=1,
                                     bytesize=8,
                                     timeout=10)
        else:
            self.ser=pipe(port[1:])
        thread.start_new_thread(self.cont_read, ())

    def waitElmPrompt(self, timeout=3):
        """waitElmPrompt(delay=3): blocks execution until ELM is ready with answer
        
        This function waits untill the cont_read thread has fully received
        a response of the ELM chip, or until a timeout.
        timeout: the timeout.
        (on a Toyota Yaris from 1999, the OBD response apears to come sometimes
        seconds after the request, that's why the default delay is 3 seconds)
        """
        startTime=time.time()
        while (self.elmState!="READY") or (time.time()>(startTime+timeout)):
            time.sleep(0.05)
        
    def cont_read(self):
        """ cont_read(): continuously read from the serial port.
        
        The ELM chip may send responses at any time, it's better to have
        one thread that reads the elm response data.
        """
        buf=""
        while True:
            c=self.ser.read(1)
            if c==">":
                self.elmState="READY"
                traceHex.log("  READ: >")
            elif c=='\0':  #ELM327DSH.pdf, page 8, note on bottum of page)
                pass
            elif c in ('\r','\n'):
                val=buf.strip()
                if(len(val)):
                    traceHex.log("  READ: "+val)
                    self.elmData+=[val]
                buf=""
            else:
                buf+=c
#    def cont_read(self):
#        i=0
#        l=""
#        while True:
#            c=self.ser.read(1)
#            if c==">":
#                print "READ: >"
#                self.elmState="READY"
#            elif c=='\0':  #ELM327DSH.pdf, page 8, note on bottum of page)
#                pass
#            else:
#                l+=c
#            nlpos=l.find("\r")
#            if nlpos==-1:
#                nlpos=l.find("\n")
#            if nlpos!=-1:
#                val=l.strip()
#                if len(val):
#                    print "READ: \""+val+"\""
#                    traceHex.log("  READ: "+val)
#                    self.elmData+=[val]
#                l=l[nlpos+1:]
#                #print "  JJJJ", l
#                while len(l) and (l[0] in ['\r', '\n']):
#                    l=l[1:]
#                #print "  JJJJ", l
                
    def send(self, s, delay=3):
        self.waitElmPrompt(delay)
        print "-> ", s
        traceHex.log("SEND: "+s)
        self.elmState="PROCESSING"
        self.ser.write(s+"\r\n")
        self.ser.flush()
        #if delay>0:
        #    time.sleep(delay)
    
    def ask(self, s, delay=3):
        self.elmData=[]
        self.send(s, delay)
        self.waitElmPrompt(delay)
        ret=self.elmData
        print "ASK:",s,"-> ", ret
        self.elmData=[]
        return ret
    
    def askOBD(self, mode, ipid, delay=3):
        s="%02x %02x"%(mode, ipid)
        try:
            data=codes.pids[mode][ipid]
            s+=" %02x"%data[0]
        except:
            pass
        ret=self.ask(s, delay)
        #ret=['41 01 02 03','41 04 05']
        answer=""
        for r in ret:
            answer+=decode.hex2bin(r)[1:]
        print "askOBD: ", hexprint(answer)
        return answer
    
    
    #print askOBD(None, None)
    #sys.exit()
    def ask_supportedPID(self, s, delay=3):
        self.askOBD()
    
            
def hexprint(s):
    return " ".join(["%02x"%ord(c) for c in s])

if __name__ == "__main__":
    ec=elmComm("|./obdtest.py")
    ec.send("ATZ")
    ec.send("ATE0")
    print hexprint(ec.askOBD("01 0C"))
    print hexprint(ec.askOBD("01 01"))
    print hexprint(ec.askOBD("01 00"))
    print decode.decodeSuppordedPID(ec.askOBD("01 00")[1:],0)
    time.sleep(0.5)
